#!/usr/local/bin/perl -w

#### LAST UPDATED: Nov15, 2002 ####
# filter2.pl version 0.1

###########################################################################
#
# This program first handles exceptions in the SOURCE file(s): contextual 
# data within square brackets and new line character within angular braces 
# in the second line of Senseval-2 test data. It does the following.
#
# a. If a new line character is found in the second line of data within 
# angular braces, the newline character is eliminated.
# b. All data comprising of multiple tokens within square brackets is assumed 
# to be contextual. This data is placed in character references 
# corresponding to square braces.
# c. All data (non-contextual) within square brackets is eliminated.
# d. Non tokenize the `\/' symbol and the pre-tag slashes `//`
# e. `\226' and `\227' tokens are removed from the data
# f. Senseval-1 data has certain non-contextual information within curly
#    braces. This data is eliminated. An exception to the rule is 
#    information about mis-spelled words. The bracketed information is 
#    replaced by the correct spelling.
#
# The processed data is stored in the DESTINATION file.
# All the data eliminated is stored in the FILTER file for reference.
#
# USAGE : filter2.pl [OPTIONS] FILTER DESTINATION SOURCE [SOURCE]... 
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

# Check if command line options have been provided
# If not print usage information 
if ( $#ARGV < 2 ) {
    &usage();  
    exit; }

# Accept DESTINATION file
$filterfl = shift(@ARGV);
$opfl = shift(@ARGV);

if(-f $opfl) {
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
        print "Overwriting the file.\n"; }
    else {
        exit; } }

if(-e $filterfl) {
    print "Filter file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
        print "Overwriting the file.\n"; }
    else {
        exit; } }

# Open filter file to store all the strings eliminated from the SOURCE
open(REM, ">$filterfl") or die "could not open file ";
# Open output file
open(OP, ">$opfl") or die "could not open file ";

# Initialize and start reading from SOURCE - line by line
$data="";
while(<>) {
    # There should be no new line character within angular braces
    # This part of code is to handle the exception in line 2 of the
    # senseval2 test data
    if(/^\s?<!DOCTYPE corpus SYSTEM\s?\n/) {
	s/^(\s?<!DOCTYPE corpus SYSTEM\s?)\n/$1/;
	$exln = <>;
	$_ = $_.$exln; }

    # All data within square brackets is assumed to be non-contextual
    # except the ones which have multiple tokens.

    # Senseval-1 data has few instances with a lone square bracket
    # float-v.800152 has token `Horrors]' which is garbled and hence removed
    s/^Horrors\] / /;
    # promise-n.800121 has a lone ] which is removed
    s/(The people have won, Aylwin is President)\]/$1/;
    # shirt-n.800409 has a lone ] which is removed
    s/(^\`No, sir)\]/$1/;

    # Multi string phrases within square braces are placed between 
    # character references representing the square braces. 
    # these phrases are assumed to be contextual information
    # s/\[(\s?[\w\.]+\s[\w\.]+[^\]]*)\]/\($1\)/g;
    # s/\[(\s*[^\]\s]+\s+[^\]\s]+\s*[^\]]*)\]/\($1\)/g;
     s/\[(\s*[^\]\s]+\s+[^\]\s]+\s*[^\]]*)\]/ &lsqb\; $1 &rsqb\; /g;
    # s/\[\s*(([^\]\s]+\s+)*)\]/ &lsqb\; $1 &rsqb\; /g;

    # Senseval-1 data has certain mis-spelled word with their 
    # correct spellings together within curly braces.
    # This data within curly braces is replaced by the correct spelling
    s/\{typo bad=\"[^\"]*\",\s*good=\"([^\"]*)\"[^\}]*\}/ $1 /g;
    
    # Print data to be eliminated in filter file
    # Square bracket data
    # while(/(\[[^\]}]*\])/g) {
      while(/([\[\{][^\]\}]*[\]\}])/g) {
	$data = $1;
	print REM "$data\n"; }	
    
    # Elimination of non-contextual data 
    s/[\[\{][^\]\}]*[\]\}]/ /g;
    
    # Non tokenize the \/ symbol
    s/\\\// /g;
    # Non tokenize the pre-tag slashes
    s/ \/ \/ /\/\//;

    # Removal of `\226' and `227' tokens
    s/\226/ /g;
    s/\227/ /g;
    
    # Print the filtered data
    print OP; }

# Subroutine to print help message
sub help {
    print "Usage: filter2.pl [OPTIONS] FILTER DESTINATION SOURCE [SOURCE]... \n\n";
    print "This program first handles exceptions in the SOURCE file(s): contextual data within square braces and new line character within angular braces in the second line of Senseval-2 test data. It does the following.\n\n";
    print "\t a. If a new line character is found in the second line of data within angular braces, the newline character is eliminated.\n";
    print "\t b. All data comprising of multiple tokens within square braces is assumed to be contextual and is placed in curved braces.\n";
    print "\t c. All data (non-contextual) within square brackets is eliminated.\n";
    print "\t d. Non tokenize the `\/' symbol and the pre-tag slashes `//`\n\n";
    print "The processed data is stored in the DESTINATION file. All the data eliminated is stored in the FILTER file for reference.\n\n";
    print "OPTIONS:\n\n";
    print "  --help             Prints the help message.\n"; 
    print "  --version          Prints the version number.\n\n"; }

sub usage {
    print "Usage: filter2.pl [OPTIONS] FILTER DESTINATION SOURCE [SOURCE]... \n"; 
    AskHelp(); }
# Message describing way to get help 
sub AskHelp {
    print "Type filter2.pl --help for help.\n"; }
# function to output the version number
sub showVersion
{
    print STDERR "filter2.pl      -        version 0.1\n";
    print STDERR "A component of posSenseval version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

